/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Sep 24, 2007
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.wsclient.samlapi;

import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;

import org.apache.xmlbeans.XmlObject;

import eu.unicore.samly2.SAMLConstants;
import eu.unicore.samly2.elements.NameID;
import eu.unicore.util.httpclient.IClientConfiguration;

import pl.edu.icm.unicore.uvos.wsclient.api.UVOSClientFactory;

/**
 * This class provides a base for SAML client implementations which use
 * SOAP binding. It adds issuer support, setups UVOS client factory and defines
 * generic methods.
 *
 * @author K. Benedyczak
 */
public abstract class SAMLVOClient extends AbstractSAMLBase
{
	protected String address;
	protected NameID issuer;
	protected boolean autoIssuer;
	protected UVOSClientFactory fact;
	
	protected XmlObject lastResponse;

	
	public SAMLVOClient(String address, IClientConfiguration secProv) 
		throws SecuritySetupException, MalformedURLException
	{
		this(address, secProv, (NameID)null);
	}

	public SAMLVOClient(String address, IClientConfiguration secProv, 
			URI issuerURI) 
		throws SecuritySetupException, MalformedURLException
	{
		this(address, secProv, new NameID(issuerURI.toString(),
				SAMLConstants.NFORMAT_ENTITY));
	}

	protected SAMLVOClient(String address, IClientConfiguration secCfg, 
			NameID issuer) 
		throws SecuritySetupException, MalformedURLException
	{
		super();
		new URL(address);
		this.address = address;
		fact = new UVOSClientFactory(secCfg);
		
		if (issuer != null)
		{
			this.issuer = issuer;
			this.autoIssuer = false;
		} else
		{
			if (secCfg.getCredential() != null)
				certificateC = secCfg.getCredential().getCertificateChain();
			if (certificateC == null || certificateC.length == 0)
				throw new SecuritySetupException("No issuer was provided" +
						" and keystore doesn't provide user's identity" +
						" so it is impossible to generate one.", null);
			this.issuer = new NameID(
				certificateC[0].getSubjectX500Principal().getName(), 
				SAMLConstants.NFORMAT_DN);
			this.autoIssuer = true;
		}
	}
	
	public XmlObject getLastResponse()
	{
		return lastResponse;
	}

	public void setIssuer(NameID issuer)
	{
		this.issuer = issuer;
		autoIssuer = false;
	}
	
	public abstract Object getProxy();
}
